/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Malte Kraus.
 *
 * Contributor(s):
 *  Malte Kraus <firefox@maltekraus.de> (Original author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function adblockFilterExporter() {
  window.addEventListener("load", this._init, false);
}
adblockFilterExporter.prototype = {
  _Uploader: function() {
    var up = Components.classes["@adblockfilters.mozdev.org/uploadjob;1"]
                       .createInstance();
    up = up.wrappedJSObject;
    up.origWindow = window;
    up.__defineGetter__("hash", function() {
      return this.origWindow.gAdblockFilterExport.hash;
    });
    up.__defineGetter__("data", function() {
      return this.origWindow.gAdblockFilterExport.data;
    });
    up.onNotUploaded = function() {
      document.documentElement.getButton("extra1").disabled = false;
    };
    return up;
  },

  uploadData: function adblockFilters__uploadData() {
    window.openDialog("chrome://adblockfilters/content/ftp-upload.xul", "",
                      "chrome", this._Uploader(), false);
  },
  _init: function adblockFilters___init() {
    var dialog = document.documentElement;
    dialog.buttons += ",extra1";
    var btn = dialog.getButton("extra1");
    btn.hidden = false;
    btn.disabled = !gAdblockFilterExport.dataChanged;

    var oldOnChange = onChange;
    onChange = function onChange() {
      oldOnChange();
      btn.disabled = !gAdblockFilterExport.dataChanged;
    };

    var appLocale = Components.classes["@mozilla.org/intl/nslocaleservice;1"]
                             .getService(Components.interfaces.nsILocaleService)
                              .getApplicationLocale();
    var strings = Components.classes["@mozilla.org/intl/stringbundle;1"]
                       .getService(Components.interfaces.nsIStringBundleService)
                            .createBundle(
                       "chrome://adblockfilters/locale/abp-settings.properties",
                                          appLocale);
    btn.setAttribute("label", strings.GetStringFromName("upload-button"));
    btn.addEventListener("command", function(event) {
      event.stopPropagation();
      applyChanges();
      gAdblockFilterExport.uploadData();
      //window.close();
      btn.disabled = true;
    }, false);
  },

  get data() {
    var abpRequired = false;
    const filters = treeView.data;

    var allRules = [];
    for(var i = 0; i < filters.length; i++) {
      if(filters[i].dummy || !filters[i].special)
        continue;

      var patterns = filters[i].patterns.slice();
      patterns.sort(sortNatural);
      for (var j = 0; j < patterns.length; j++) {
        var pattern = patterns[j].orig.text;

        if(pattern in treeView.disabled)
          pattern = "!" + pattern;

        allRules.push(pattern);

        switch(patterns[j].orig.type) {
          case "whitelist":
          case "elemhide":
            abpRequired = true;
            break;
          case "filterlist":
            if(pattern[0] == "|")
              abpRequired = true;
            else if(pattern[pattern.length - 1] == "|")
              abpRequired = true;
            else if(pattern.indexOf("$") > -1)
              abpRequired = true;
            break;
        }
      }

    }

    var data = "[Adblock" + (abpRequired ? " Plus" : "") + "]\n";
    data += allRules.join("\n");
    return data;
  },

  get hash() {
    var converter = Components.classes
                                ["@mozilla.org/intl/scriptableunicodeconverter"]
                                .createInstance(Components.interfaces
                                                .nsIScriptableUnicodeConverter);

    converter.charset = "UTF-8";
    var data = converter.convertToByteArray(this.data, {});

    var ch = Components.classes["@mozilla.org/security/hash;1"]
                       .createInstance(Components.interfaces.nsICryptoHash);
    ch.init(ch.SHA1);
    ch.update(data, data.length);
    var hash = ch.finish(false);

    // convert the binary hash data to a hex string
    var hexString = "";
    for(var i in hash)
      hexString += ("0" + hash.charCodeAt(i).toString(16)).slice(-2);

    return hexString;
  },

  get dataChanged() {
    const PREF = "extensions.adblockfilters.lastUploadHash";
    var hash = this.hash;
    return (nsPreferences.copyUnicharPref(PREF) != hash);
  },

  showPreferences: function adblockFilters__showPreferences() {
    window.openDialog("chrome://adblockfilters/content/settings.xul", "", "chrome,dialog,resizable,dependent=0,modal=0");
  }
}
var gAdblockFilterExport = new adblockFilterExporter();